
/*---------------------------------------------------------------------------
    uread -- reads in a Fortran unformatted file, and prints out 
             the lengths of the data segments. The -s option flags
             byte swaping of the input lengths, for files written
	     on other sex computers.

    Written By: S.E. Norris
  ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

static char usage[] = "Usage: %s [-x] [filename]\n";
void pusage(char *);

typedef union {
  char ch[4];
  int  i;
} swap;

main(int argc, char *argv[])
{
  int i, c, alph, ls, le;
  int transsex = 0;
  swap l, m;
  FILE *in;
  struct stat stat;
  
  in = stdin;
  i = 1;

  if (argc > 3) pusage(argv[0]);
  while (i < argc) {
    if (strcmp(argv[i],"-x") == 0) {
      transsex = 1;
    } else if (argv[i][0] != '-') {
      if ((in = fopen(argv[i],"r")) == NULL) {
	fprintf(stderr,"%s: %s",argv[0],argv[i]);
	perror(" ");
	exit(EXIT_FAILURE);
      }
    } else {
      pusage(argv[0]);
    }
    i++;
  }


  /*
   *  Get stats on the file input
   */
  if (in != stdin)
  {
    if (fstat(fileno(in),&stat) != 0) {
      perror("fstat");
      exit(EXIT_FAILURE);
    }
  }

  for (;;) {

    /*
     *  Get length of data segment
     */
    for (i=0;i<4;i++) {
      if ((l.ch[i] = fgetc(in)) == EOF) {
	exit(0);
      }
    }

    /*
     *  Swap bytes if called to do so
     */
    if (transsex) {
      m.ch[0] = l.ch[3];
      m.ch[1] = l.ch[2];
      m.ch[2] = l.ch[1];
      m.ch[3] = l.ch[0];
      ls = m.i;
    } else {
      ls = l.i;
    }

    /*
     *  Check length makes sence
     */
    if (ls < 0) {
      fprintf(stderr,"%s: negative data size\n",argv[0]);
      exit(EXIT_FAILURE);
    }
    if (in != stdin)
    {
      if (ls > stat.st_size) {
        fprintf(stderr,"%s: data larger than file length\n",argv[0]);
        exit(EXIT_FAILURE);
      }
    }

    /*
     *  Read in data, and check that ascii
     */
    alph = 1;
    for (i=0;i<ls;i++) {
      if ((c = fgetc(in)) == EOF) {
	perror("fgetc");
	exit(EXIT_FAILURE);
      }
      if (!(isgraph(c) || isspace(c))) {
	alph = 0;
      }
    }

    /*
     *  Read trailing flag -- compare with leading flag
     */
    for (i=0;i<4;i++) {
      if ((l.ch[i] = fgetc(in)) == EOF) {
	exit(EXIT_SUCCESS);
      }
    }
    
    /*
     *  Swap bytes if called to do so
     */
    if (transsex) {
      m.ch[0] = l.ch[3];
      m.ch[1] = l.ch[2];
      m.ch[2] = l.ch[1];
      m.ch[3] = l.ch[0];
      le = m.i;
    } else {
      le = l.i;
    }

    if (ls != le) {
      fprintf(stderr,"%s: data lengths not equal\n",argv[0]);
      exit(EXIT_FAILURE);
    }

    /* Print out data */
    printf("%10.d :",ls);
    if (alph) printf(" char");
    if ((ls %  2) == 0) printf(" (%d)*2",ls/2);
    if ((ls %  4) == 0) printf(" (%d)*4",ls/4);
    if ((ls %  8) == 0) printf(" (%d)*8",ls/8);
    if ((ls % 16) == 0) printf(" (%d)*16",ls/16);
    printf("\n");
  }
}


void pusage(char *name)
{
  fprintf(stderr,usage,name);
  exit(EXIT_FAILURE);
}
