
/*---------------------------------------------------------------------------
    ustrip -- strips record data from a Fortran unformatted file.

    Written By: S.E. Norris
  ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <signal.h>

static char usage[] = "Usage: %s [-x] [filename]\n";
void pusage(char *);
FILE *in, *out;
static char infile[FILENAME_MAX];
static char outfile[FILENAME_MAX];

typedef union {
  char ch[4];
  int  i;
} swap;


main(int argc, char *argv[])
{
  int i, c, ls, le;
  int transsex = 0;
  swap l, m;
  struct stat stat;
  mode_t mode;
  struct utimbuf times;
  uid_t owner;
  gid_t group;
  extern void quit();
  
  in  = stdin;
  out = stdout;
  i = 1;

  if (argc > 3) pusage(argv[0]);
  while (i < argc) {
    if (strcmp(argv[i],"-x") == 0) {
      transsex = 1;
    } else if (argv[i][0] != '-') {
      /*
       * Get input and (temp) output filenames, and open them.
       * Set up signals to delete files if program suspended
       */
      strcpy(infile,argv[i]);
      strcpy(outfile,argv[i]);
      strcat(outfile,"tmp");
      signal(SIGINT,  quit);
      signal(SIGQUIT, quit);
      signal(SIGTERM, quit);
      if ((in = fopen(argv[i],"rw")) == NULL) {
	fprintf(stderr,"%s: %s",argv[0],argv[i]);
	perror(" ");
	exit(EXIT_FAILURE);
      }
      if ((out = fopen(outfile,"w")) == NULL) {
        fprintf(stderr,"%s: could not create %s\n",argv[0],outfile);
        perror(" ");
        exit(EXIT_FAILURE);
      }
    } else {
      pusage(argv[0]);
    }
    i++;
  }


  /*
   *  Get stats on the file input
   */
  if (in != stdin)
  {
    if (fstat(fileno(in),&stat) != 0) {
      perror("fstat");
      exit(EXIT_FAILURE);
    }
  }
  mode = stat.st_mode;
  times.actime  = stat.st_atime;
  times.modtime = stat.st_mtime;
  owner = stat.st_uid;
  group = stat.st_gid;
   
  while(1) {

    /*
     *  Get length of data segment
     */
    for (i=0;i<4;i++) {
      if ((l.ch[i] = fgetc(in)) == EOF) {

	/*
	 * If we are at the end of the file, and we arn't reading from
	 * standard input, close files and overwrite the input file
	 */
	if (in == stdin) {
	  exit(EXIT_SUCCESS);
	}
	if ((fclose(in) != 0) | (fclose(out) != 0)) {
	  perror("close");
          exit(EXIT_FAILURE);
	}
        if (rename(outfile,infile) != 0) {
          perror("rename");
          exit(EXIT_FAILURE);
	}
	if (chmod(infile,mode)   != 0) perror("chmod");
	chown(infile,owner,group);
	if (utime(infile,&times) != 0) perror("utimes");
        exit(EXIT_SUCCESS);
      }
    }

    /*
     *  Swap bytes if called to do so
     */
    if (transsex) {
      m.ch[0] = l.ch[3];
      m.ch[1] = l.ch[2];
      m.ch[2] = l.ch[1];
      m.ch[3] = l.ch[0];
      ls = m.i;
    } else {
      ls = l.i;
    }

    /*
     *  Check length makes sence
     */
    if (ls < 0) {
      fprintf(stderr,"%s: negative data size\n",argv[0]);
      exit(EXIT_FAILURE);
    }
    if (ls == 0) {
      fprintf(stderr,"%s: zero data size!\n",argv[0]);
    }
    if (in != stdin)
    {
      if (ls > stat.st_size) {
        fprintf(stderr,"%s: data larger than file length\n",argv[0]);
        exit(EXIT_FAILURE);
      }
    }

    /*
     *  Read in data, and check that ascii
     */
    for (i=0;i<ls;i++) {
      if ((c = fgetc(in)) == EOF) {
	perror("fgetc");
	exit(EXIT_FAILURE);
      }
      if (fputc(c,out) == EOF) {
        perror("fputc");
        exit(EXIT_FAILURE);
      }
    }

    /*
     *  Read trailing flag -- compare with leading flag
     */
    for (i=0;i<4;i++) {
      if ((l.ch[i] = fgetc(in)) == EOF) {
	exit(EXIT_SUCCESS);
      }
    }
    
    /*
     *  Swap bytes if called to do so
     */
    if (transsex) {
      m.ch[0] = l.ch[3];
      m.ch[1] = l.ch[2];
      m.ch[2] = l.ch[1];
      m.ch[3] = l.ch[0];
      le = m.i;
    } else {
      le = l.i;
    }

    if (ls != le) {
      fprintf(stderr,"%s: data lengths not equal\n",argv[0]);
      exit(EXIT_FAILURE);
    }
  }
}


void pusage(char *name)
{
  fprintf(stderr,usage,name);
  exit(EXIT_FAILURE);
}


void quit(int i)
{
  if (in == stdin) exit(EXIT_FAILURE);

  if (unlink(outfile) != 0) {
    perror("unlink");
  }
  exit(EXIT_FAILURE);
}


