
/*---------------------------------------------------------------------------
    uswap -- reads in a Fortran unformatted file, and prints out 
             the data with the records desexed.

    Written By: S.E. Norris
  ---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

static char usage[] = "Usage: %s [-x] [-csfde...] [-i filename] [-o filename]\n";
void pusage(char *);

typedef union {
  char   c[16];
  int    i[4];
} swap;

main(int argc, char *argv[])
{
  int i, j, k = 0, ls, le, s, zog;
  int transsex = 0;
  swap l, m, buff;
  FILE *in, *out;
  struct stat sd;
  char *types;
  char type = 'f';


  types = "-f";
  in  = stdin;
  out = stdout;
  i = 1;

  if (argc > 7) pusage(argv[0]);
  while (i < argc) {
    if (strcmp(argv[i],"-i") == 0) {
      if (++i >= argc) pusage(argv[0]);
      if ((in = fopen(argv[i],"r")) == NULL) {
	fprintf(stderr,"%s: %s",argv[0],argv[i]);
	perror(" ");
	exit(EXIT_FAILURE);
      }
    } else if (strcmp(argv[i],"-o") == 0) {
      if (++i >= argc) pusage(argv[0]);
      if ((out = fopen(argv[i],"w")) == NULL) {
	fprintf(stderr,"%s: %s",argv[0],argv[i]);
	perror(" ");
	exit(EXIT_FAILURE);
      }
    } else if (strcmp(argv[i],"-x") == 0) {
      transsex = 1;
    } else if (argv[i][0] == '-') {
      if (argv[i][1] != '\n') {
	types = argv[i];
      } else {
	pusage(argv[0]);
      }
    } else {
      pusage(argv[0]);
    }
    i++;
  }


  /*
   *  Get stats on the file input
   */
  if (in != stdin) {
    if (fstat(fileno(in),&sd) != 0) {
      perror("fstat");
      exit(EXIT_FAILURE);
    }
  }

  for (;;) {

    /*
     *  Get type from type string
     */
    switch (types[k+1]) {
    case 'c':
      s = 1;
      k++;
      type = types[k];
      break;
    case 's':
      s = 2;
      k++;
      type = types[k];
      break;
    case 'f':
      s = 4;
      k++;
      type = types[k];
      break;
    case 'd':
      s = 8;
      k++;
      type = types[k];
      break;
    case 'e':
      s = 16;
      k++;
      type = types[k];
      break;
    case '\0':
      break;
    default:
      fprintf(stderr,"%s: unknown data type %c\n",argv[0],types[k+1]);
      pusage(argv[0]);
      break;
    }
 
    /*
     *  Get length of data segment, swap bytes
     */
    for (i=0;i<4;i++) {
      if ((l.c[i] = (char) fgetc(in)) == EOF) {
	exit(0);
      }
    }
    m.c[0] = l.c[3];
    m.c[1] = l.c[2];
    m.c[2] = l.c[1];
    m.c[3] = l.c[0];

    if (transsex) {
      ls = m.i[0];
    } else {
      ls = l.i[0];
    }
    for (i=0;i<4;i++) {
      fputc(m.c[i],out);
    }

    /*
     *  Check length makes sence
     */
    if (ls < 0) {
      fprintf(stderr,"%s: negative data size\n",argv[0]);
      exit(EXIT_FAILURE);
    }
    if (in != stdin)
    {
      if (ls > sd.st_size) {
        fprintf(stderr,"%s: data larger than file length\n",argv[0]);
        exit(EXIT_FAILURE);
      }
    }
    if (ls%s != 0) {
      fprintf(stderr,"%s: field nor divisable by %d\n",argv[0],s);
      exit(EXIT_FAILURE);
    }

    /*
     *  Read in data, swap bytes
     */
    for (i=0;i<(ls/s);i++) {
      for (j=0;j<s;j++) {
	if ((zog = fgetc(in)) == EOF) {
	  perror("fgetc");
	  exit(1);
	}
	buff.c[j] = zog;
      }
      for (j=s;j>0;j--) {
	fputc(buff.c[j-1],out);
      }
    }

    /*
     *  Read trailing flag -- compare with leading flag
     */
    for (i=0;i<4;i++) {
      if ((l.c[i] = (char) fgetc(in)) == EOF) {
	exit(0);
      }
    }
    m.c[0] = l.c[3];
    m.c[1] = l.c[2];
    m.c[2] = l.c[1];
    m.c[3] = l.c[0];

    if (transsex) {
      le = m.i[0];
    } else {
      le = l.i[0];
    }
    for (i=0;i<4;i++) {
      fputc(m.c[i],out);
    }
    if (ls != le) {
      fprintf(stderr,"%s: data lengths not equal\n",argv[0]);
      exit(1);
    }

    /*
     *  Print out data
     */
    fprintf(stderr,"%10.d :",ls/s);
    switch (type) {
    case 'c':
      fprintf(stderr," char\n");
      break;
    case 's':
      fprintf(stderr," short\n");
      break;
    case 'f':
      fprintf(stderr," float\n");
      break;
    case 'd':
      fprintf(stderr," double\n");
      break;
    case 'e':
      fprintf(stderr," extended\n");
      break;
    }
  }
}


void pusage(char *name)
{
  fprintf(stderr,usage,name);
  exit(EXIT_FAILURE);
}
